// u_30_profiling
// 2013-02-09

// -- (C) John COLIBRI 2013
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_30_profiling;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    exit_: TButton;
    clear_: TButton;
    Panel3: TPanel;
    Memo1: TMemo;
    compare_calls_: TButton;
    measure_anonymous_: TButton;
    global_object_call_: TButton;
    procedure exit_Click(Sender: TObject);
    procedure clear_Click(Sender: TObject);
    procedure compare_calls_Click(Sender: TObject);
    procedure measure_anonymous_Click(Sender: TObject);
    procedure eClick(Sender: TObject);
    procedure dClick(Sender: TObject);
    procedure cClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure global_object_call_Click(Sender: TObject);
  private
  public
  end;

var
  Form1: TForm1;

implementation
    uses u_display_simple, u_c_profile, u_c_to_profile;

    {$R *.dfm}

    procedure TForm1.FormCreate(Sender: TObject);
      begin
        initialize_display(Memo1.Lines);
        Left:= 170; Top:= 110;
      end; // FormCreate

    procedure TForm1.clear_Click(Sender: TObject);
      begin
        Memo1.Lines.Clear;
      end; // clear_Click

    procedure TForm1.exit_Click(Sender: TObject);
      begin
        Close;
      end; // exit_Click

    procedure TForm1.compare_calls_Click(Sender: TObject);
      const k_recursive_count= 10000;
      var l_c_procedure_timer: c_procedure_timer;
          l_c_my_object: c_my_class;
          l_i_my_interface: i_my_interface;
      begin
        display('');
        l_c_procedure_timer:= c_procedure_timer.Create;

        try
          l_c_procedure_timer.m_initial_iterations:= 100;
          l_c_procedure_timer.m_iterations:= 100;

          l_c_my_object:= c_my_class.Create;
          l_i_my_interface:= l_c_my_object;

          display('');
          l_c_procedure_timer.measure_time('procedure(n)',
              procedure
                begin
                  recursive_call(k_recursive_count);
                end
              );

          display('');
          l_c_procedure_timer.measure_time('l_c.method(p_c, n)',
              procedure
                begin
                  l_c_my_object.non_virtual_call(l_c_my_object, k_recursive_count);
                end
              );
          l_c_procedure_timer.measure_time('l_c.method(const p_c, n)',
              procedure
                begin
                  l_c_my_object.const_non_virtual_call(l_c_my_object, k_recursive_count);
                end
              );

          display('');
          l_c_procedure_timer.measure_time('l_c.method(p_c, n) Virt',
              procedure
                begin
                  l_c_my_object.virtual_call(l_c_my_object, k_recursive_count);
                end
              );

          display('');
          l_c_procedure_timer.measure_time('l_i.method(p_i, n)',
              procedure
                begin
                  l_i_my_interface.interface_call(l_i_my_interface, k_recursive_count);
                end
              );
          l_c_procedure_timer.measure_time('l_i.method(const p_i, n)',
              procedure
                begin
                  l_i_my_interface.const_interface_call(l_i_my_interface, k_recursive_count);
                end
              );

        finally
          l_c_procedure_timer.Free;
        end;
      end; // compare_calls_Click

    procedure TForm1.measure_anonymous_Click(Sender: TObject);
      const k_recursive_count= 10000;
      var l_c_procedure_timer: c_procedure_timer;
          l_ap: t_ap;
          l_ap_const: t_ap_const;
      begin
        display('');
        l_c_procedure_timer:= c_procedure_timer.Create;

        try
          l_c_procedure_timer.m_initial_iterations:= 100;
          l_c_procedure_timer.m_iterations:= 100;

          l_ap:=
              procedure (p_ap: t_ap; p_count: Integer)
                begin
                  if p_count> 0
                    then p_ap(p_ap, p_count- 1);
                end;

          l_ap_const:=
              procedure (const pk_ap: t_ap_const; p_count: Integer)
                begin
                  if p_count> 0
                    then pk_ap(pk_ap, p_count- 1);
                end;
          display('');

          l_c_procedure_timer.measure_time('anon(p_anon, n)',
              procedure
                begin
                  l_ap(l_ap, k_recursive_count);
                end
              );
          l_c_procedure_timer.measure_time('anon(const p_anon, n)',
              procedure
                begin
                  l_ap_const(l_ap_const, k_recursive_count);
                end
              );
        finally
          l_c_procedure_timer.Free;
        end;
      end; // measure_anonymous_Click

    procedure TForm1.global_object_call_Click(Sender: TObject);
      const k_recursive_count= 10000;
      var l_c_procedure_timer: c_procedure_timer;
      begin
        display('');
        l_c_procedure_timer:= c_procedure_timer.Create;

        try
          l_c_procedure_timer.m_initial_iterations:= 100;
          l_c_procedure_timer.m_iterations:= 100;

          g_c_my_object:= c_my_class.Create;
          display('');

          l_c_procedure_timer.measure_time('g_c.call(n)',
              procedure
                begin
                  g_c_my_object.non_virtual_global_call(k_recursive_count);
                end
              );
        finally
          l_c_procedure_timer.Free;
        end;
      end; // global_object_call_Click

procedure TForm1.cClick(Sender: TObject);
      begin
      end; //

    procedure TForm1.dClick(Sender: TObject);
      begin
      end; //

    procedure TForm1.eClick(Sender: TObject);
      begin
      end; //

    end.

