// u_30_profiling
// 2013-02-09

// -- (C) John COLIBRI 2013
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_profile;
  interface
    uses sysutils;

    const k_default_iterations= 3;
          k_default_overhead= 1;

    type c_procedure_timer=
             class
               m_initial_iterations: Integer;
               m_iterations: Integer;

               m_base_frequency: Int64;

               constructor Create;

               procedure read_hires_frequency;

               function f_measure_time_of(
                   p_pa_to_profile: TProc;
                   p_initial_iterations, p_iterations: Integer): Double;

               procedure measure_time(
                   const p_title: string;
                   const p_pa_to_profile: TProc); overload;
             end; // c_procedure_timer

  implementation
    uses Windows, u_display_simple;

    { c_procedure_timer }

    constructor c_procedure_timer.Create;
      begin
        read_hires_frequency;

        m_initial_iterations:= k_default_overhead;
        m_iterations:= k_default_iterations;

        // -- try to eliminate initial setup time
        f_measure_time_of(
            procedure
              begin
              end,
            100, 3);
      end; // Create

    procedure c_procedure_timer.read_hires_frequency;
      begin
        if (m_base_frequency= 0) and not QueryPerformanceFrequency(m_base_frequency)
          then raise Exception.Create('No high-performance counter available.');
      end; // read_hires_frequency

    function c_procedure_timer.f_measure_time_of(
        p_pa_to_profile: TProc;
        p_initial_iterations, p_iterations: Integer): Double;
      var l_start_ticks, l_stop_ticks: Int64;
          l_iteration: Integer;
      begin
        // -- try to eliminate initial setup time
        for l_iteration:= 1 to p_initial_iterations do
          p_pa_to_profile;

        QueryPerformanceCounter(l_start_ticks);

        for l_iteration:= 1 to p_iterations do
          p_pa_to_profile;

        QueryPerformanceCounter(l_stop_ticks);

        Result:= (l_stop_ticks- l_start_ticks)/ m_base_frequency/ p_iterations* 1000;
      end; // f_measure_time_of

    procedure c_procedure_timer.measure_time(
        const p_title: string;
        const p_pa_to_profile: TProc);
      var l_time: Double;
      begin
        l_time:= f_measure_time_of(p_pa_to_profile,
            m_initial_iterations, m_iterations);

        display(Format('%-30s %15.9f ms', [p_title, l_time]));
      end; // measure_time

    end.
